
%  Initialization
clear
addPath();
[dbs, datamat, strs, num, criway, resultsFolder] = setConfigurations();
resultPath = ['./data/' resultsFolder];
if ~exist(resultPath,'dir')
    mkdir(resultPath);
end

% Load data
load(datamat);

% Semi-supervised learning
for dbnum = 1:length(dbs)
    writePath = [resultPath '/' dbs{dbnum}];
    if ~exist(writePath,'dir')
        mkdir(writePath);
    end
    dbnum2 = str2num(dbs{dbnum}(3:end));
    AData_db = ADN{dbnum2}; BData_db = BDN{dbnum2}; 
    indeData_db = IDN{dbnum2};
    CData_db = CDC{dbnum2};      
    
    for sNum = 1:length(strs)
        switch strs{sNum}
            case 'onlyA'
                % 1. Train a classifier (onlyA) using A, and test it on IDN.
                writePath_only = [writePath '/classifier_onlyA.mat'];
                if ~exist(writePath_only,'file')
                    model_onlyA = trainClassifier(AData_db,ADN_target,'onlyA',writePath,criway);
                    [evalCriteria_onlyA, outputs_onlyA,~] = testClassifier(model_onlyA,indeData_db,IDN_target,criway);
                    save(writePath_only, 'model_onlyA','evalCriteria_onlyA','outputs_onlyA');
                else
                    load(writePath_only, 'model_onlyA');
                end
            case 'onlyB'
                % 2. Train a classifier (onlyB) using B, and test it on IDN.
                writePath_only = [writePath '/classifier_onlyB.mat'];
                if ~exist(writePath_only,'file')
                    model_onlyB = trainClassifier(BData_db,BDN_target,'onlyB',writePath,criway);
                    [evalCriteria_onlyB, outputs_onlyB,~] = testClassifier(model_onlyB,indeData_db,IDN_target,criway);
                    save(writePath_only, 'model_onlyB','evalCriteria_onlyB','outputs_onlyB');
                end
            case 'AB'
                % 3. Train a classifier (AB) using A+B, and test it on IDN.
                writePath_AB = [writePath '/classifier_AB.mat'];
                if ~exist(writePath_AB,'file')
                    model_AB = trainClassifier([AData_db;BData_db],[ADN_target,BDN_target],'AB',writePath,criway);
                    [evalCriteria_AB, outputs_AB,~] = testClassifier(model_AB,indeData_db,IDN_target,criway);
                    save(writePath_AB, 'model_AB','evalCriteria_AB','outputs_AB');
                end
            case 'AsemiB'
                % 4. Take ADN as initial training set, and add BDN using
                % semi-supervised learning. Use one-classifier mode train a
                % predicotr (AsemiB), and test it on IDN.
                writePath_AsemiB = [writePath '/AsemiB'];
                if ~exist(writePath_AsemiB,'dir')
                    mkdir(writePath_AsemiB);
                end
                if ~exist([writePath_AsemiB '/done.mat'],'file')
                    trainSemiClassifier(model_onlyA,AData_db,ADN_target,BData_db,BDN_target,...
                        indeData_db,IDN_target,writePath_AsemiB,criway); 
                end
            case 'AnsemiB'
                % 5. Take ADN as initial training set, and add BDN using
                % semi-supervised learning. Use two-classifier mode or three-classifier
                % mode train a predictor (A2semiB or A3semiB),and test it
                % on IDN.
                str = ['A' num2str(num) 'semiB'];  % A2semiB or A3semiB
                writePath_AnsemiB = [writePath '/' str];
                if ~exist(writePath_AnsemiB,'dir')
                    mkdir(writePath_AnsemiB);
                end
                if ~exist([writePath_AnsemiB '/done.mat'],'file')
                    trainSemiClassifier_nClasf(AData_db,ADN_target,BData_db,BDN_target,indeData_db,...
                        IDN_target,writePath_AnsemiB,num,criway);
                end
            case 'AsemiBC'
                % 6. As the follow-up work of AsemiB, we adds CDC to
                % the training set of AsemiB using semi-supervised
                % learning. Use one-classifier mode train a predictor
                % (AsemiBC), and test it on IDN.
                writePath_AsemiBdone = [writePath '/AsemiB/done.mat'];
                if ~exist(writePath_AsemiBdone,'file')
                    error('Please complete the AsemiB predictor.');
                end
                % load the results of AsemiB iteration
                load(writePath_AsemiBdone);
                bestiter = size(done,2);
                load([writePath '/AsemiB/markNum.mat']);
                BData_db_best = BData_db(markNum{1,bestiter},:);
                BDN_target_best = BDN_target(:,markNum{1,bestiter});
                % add CDC data
                writePath_AsemiBC = [writePath '/AsemiBC'];
                if ~exist(writePath_AsemiBC,'dir')
                    mkdir(writePath_AsemiBC);
                end
                if ~exist([writePath_AsemiBC '/done.mat'],'file')
                    load([writePath '/AsemiB/classifier_AsemiB_' num2str(bestiter) '.mat'],'model_AsemiB');
                    trainSemiClassifier(model_AsemiB,[AData_db;BData_db_best],...
                        [ADN_target,BDN_target_best],CData_db,CDC_normalTarget,indeData_db,...
                        IDN_target,writePath_AsemiBC,criway);
                end
            case 'AnsemiBC'
                % 7. As the follow-up work of A2semiB or A3semiB, we adds CDC to
                % the training set of A2semiB or A3semiB using semi-supervised
                % learning. Use two-classifier mode or three-classifier mode train a predictor
                % (A2semiBC or A3semiBC), and test it on IDN.
                str = ['A' num2str(num) 'semiBC']; % A2semiBC or A3semiBC
                writePath_AnsemiBC = [writePath '/' str];
                if ~exist(writePath_AnsemiBC,'dir')
                    mkdir(writePath_AnsemiBC);
                end
                if ~exist([writePath_AnsemiBC '/done.mat'],'file')
                    trainSemiClassifier_nClasf({AData_db,BData_db},[ADN_target BDN_target],...
                        CData_db,CDC_normalTarget,indeData_db,IDN_target,writePath_AnsemiBC,...
                        num,criway);
                end
            otherwise
                error('Please input the correct strs.');
        end
    end
end